@@ -48,6 +48,107 @@ def fixNonWatertight(mesh):
 
     return False
 
+def removeEyeBalls(mesh):
+    # get max z vertex
+    vertex_z_max_index = mesh.vertices[:, 2].argmax()
+
+    def get_vertices_connect_vertex(mesh, vertex, path=[]):
+        path += [vertex]
+
+        for neighbor in list(mesh.vertex_adjacency_graph[vertex]):
+            if neighbor not in path:
+                path = get_vertices_connect_vertex(mesh, neighbor, path)
+
+        return path
+
+    vertices_connect = get_vertices_connect_vertex(mesh, vertex_z_max_index)
+
+    vertices = np.array(mesh.vertices)
+    faces = np.array(mesh.faces)
+    
+    face_index = []
+    for v_c in vertices_connect:
+        face_index += list(np.where(faces == v_c)[0])
+    
+    face_index = list(set(face_index))
+
+    return trimesh.Trimesh(
+    vertices=vertices,
+    faces=faces[face_index]
+    )
+
+def get_faces_by_edge(u_index, v_index, mesh_faces):
+    u_faces = np.where(mesh_faces == u_index)[0]
+    v_index = np.where(mesh_faces == v_index)[0]
+
+    return np.intersect1d(u_faces, v_index)
+
+def fixNonWatertight_1vertex(mesh):
+    non_watertight_edge = []
+    for vertex in range(len(mesh.vertices)):
+        for neighbor in list(mesh.vertex_adjacency_graph[vertex]):
+            colab_faces = get_faces_by_edge(vertex, neighbor, mesh.faces)
+            if len(colab_faces) == 1:
+                if ([vertex, neighbor] not in non_watertight_edge) and ([vertex, neighbor][::-1] not in non_watertight_edge):
+                    non_watertight_edge.append([vertex, neighbor])
+
+    temp_non_watertight_edge = non_watertight_edge.copy()
+    hold_rings = []
+
+    hold_rings.append(temp_non_watertight_edge.pop())
+    hold_rings_index = 0
+    loop = True
+    while loop:
+        flat = 0
+        ring = hold_rings[hold_rings_index] 
+
+        check_first = np.where(np.array(temp_non_watertight_edge) == ring[0])
+        if len(check_first[0]) > 0:
+            vertex = temp_non_watertight_edge[check_first[0][0]][check_first[1][0]-1]
+            ring.insert(0, vertex)
+            del temp_non_watertight_edge[check_first[0][0]]
+        else:
+            flat += 1
+
+        check_last = np.where(np.array(temp_non_watertight_edge) == ring[1])
+        if len(check_last[0]) > 0:
+            vertex = temp_non_watertight_edge[check_last[0][0]][check_last[1][0]-1]
+            ring.append(0, vertex)
+            del temp_non_watertight_edge[check_last[0][0]]
+        else:
+            flat += 1
+
+        # print(check_first, check_last)
+        hold_rings[hold_rings_index] = ring
+
+        if flat == 2 and len(temp_non_watertight_edge) > 0:
+            hold_rings.append(temp_non_watertight_edge.pop())
+            hold_rings_index += 1
+        elif len(temp_non_watertight_edge) == 0:
+            loop = False
+    
+
+    new_mesh = mesh.copy()
+    for ring in hold_rings:
+        mesh_faces = np.array(new_mesh.faces)
+        mesh_vertices = np.array(new_mesh.vertices)
+        
+        ring = ring[1:]
+        mid_point = np.array(np.sum(new_mesh.vertices[ring], axis=0))/len(ring)
+        new_mesh_vertices = np.append(mesh_vertices, [mid_point], axis=0)
+
+        hole_faces = []
+        for idx in range(0, len(ring)):
+            hole_faces.append([ring[idx], len(new_mesh_vertices)-1, ring[idx-1]])
+
+        new_mesh_faces = np.append(mesh_faces, hole_faces, axis=0)
+
+        new_mesh = trimesh.Trimesh(new_mesh_vertices, new_mesh_faces)
+
+    if new_mesh.is_watertight == True:
+        return new_mesh
+
+    return False
 
 ############################
 # Increasing mesh vertices #
@@ -781,7 +882,7 @@ def func_FrontFace(mesh):
 def removedup(x):
   return list(dict.fromkeys(x))
 
-def circleScar(mesh, path, list_face_of_vertice, limitY_min, limitY_max, index):
+def circleScar(mesh, index, path, list_face_of_vertice, limitY_min, limitY_max):
     mesh_pre = mesh.copy()
 
     #danh sách index face sẽ lấy random
@@ -791,8 +892,8 @@ def circleScar(mesh, path, list_face_of_vertice, limitY_min, limitY_max, index):
             if func_checkY(mesh_pre.vertices,mesh_pre.faces[i], limitY_min, limitY_max):
                 list_index_face.append(i)
     index_Face_begin = np.random.choice(list_index_face)
-    min_num_point = int(len(list_index_face)/100)
-    max_num_point = int(len(list_index_face)/20)
+    min_num_point = int(len(list_index_face)/50)
+    max_num_point = int(len(list_index_face)/10)
     num_vertices_pick = np.random.randint(min_num_point,max_num_point)
     lst_vertices_picked = []
     lst_vertices_picked.append(mesh_pre.faces[index_Face_begin][0])
@@ -820,8 +921,8 @@ def circleScar(mesh, path, list_face_of_vertice, limitY_min, limitY_max, index):
         _tmp_ver_pkd.append(_tmp_ver)
 
     # mesh_pre1 = mesh_pre.copy()
-    _inc_Z = np.random.randint(4,12)
-    llZ = np.random.randint(0,2)
+    _inc_Z = np.random.randint(20,30)
+    llZ = np.random.randint(0,1)
     sm = len(_tmp_ver_pkd)
     for i in range(sm):
         for idx_ver in _tmp_ver_pkd[i]:
@@ -829,21 +930,120 @@ def circleScar(mesh, path, list_face_of_vertice, limitY_min, limitY_max, index):
                 mesh_pre.vertices[idx_ver][2] =  mesh_pre.vertices[idx_ver][2] + _inc_Z*(sm - i)/sm
             else:
                 mesh_pre.vertices[idx_ver][2] =  mesh_pre.vertices[idx_ver][2] - _inc_Z*(sm - i)/sm
-    newpath = path + "_" + str(index) + ".obj"
-    mesh_pre.export(newpath)
-    # return mesh_pre
+
+    # newpath = path + "_" + str(index) + ".obj"
+    # mesh_pre.export(newpath)
+
+    if index != None:
+        newpath = path + "_" + str(index) + ".obj"
+        mesh_pre.export(newpath)
+    elif path != None:
+        newpath = path + ".obj"
+        mesh_pre.export(newpath)
+
+    return mesh_pre
+
+
+# lặp chỗ này để tạo sẹo dài
+def longScar(mesh, index, path, list_face_of_vertice, limitY_min, limitY_max):
+    mesh_pre = mesh.copy()
+
+    #danh sách index face sẽ lấy random
+    list_index_face = []
+    for i in range(0, len(mesh_pre.faces)):
+        if func_checkZ(mesh_pre.vertices,mesh_pre.faces[i]):
+            if func_checkY(mesh_pre.vertices,mesh_pre.faces[i], limitY_min, limitY_max):
+                list_index_face.append(i)
+    index_Face_begin = np.random.choice(list_index_face)
+    min_num_point = int(len(list_index_face)/200)
+    max_num_point = int(len(list_index_face)/20)
+    num_vertices_pick = np.random.randint(min_num_point,max_num_point)
+    lst_vertices_picked = []
+    lst_vertices_picked.append(mesh_pre.faces[index_Face_begin][0])
+    lst_vertices_picked.append(mesh_pre.faces[index_Face_begin][1])
+    lst_vertices_picked.append(mesh_pre.faces[index_Face_begin][2])
+    _tmp_ver_pkd = []
+    _tmp_ver_pkd.append(  lst_vertices_picked.copy())
+    randXY = np.random.randint(0,2)
+    while(len(lst_vertices_picked) < num_vertices_pick):
+        _tmp_ver = []
+        for idx_ver in _tmp_ver_pkd[-1]:
+            lst_face_of_ver = list_face_of_vertice[idx_ver]
+            begin_lstfov = 0
+
+            for idx_f in range(begin_lstfov, len( lst_face_of_ver)):
+                faceofv = lst_face_of_ver[idx_f]
+                _tmp_ver.append(mesh_pre.faces[faceofv][0])
+                _tmp_ver.append(mesh_pre.faces[faceofv][1])
+                _tmp_ver.append(mesh_pre.faces[faceofv][2])
+        _tmp_ver = removedup(_tmp_ver)
+        lst_cood_ver_X = []
+        for idx_c_v in _tmp_ver:
+            lst_cood_ver_X.append([mesh_pre.vertices[idx_c_v][randXY],idx_c_v])
+
+        lst_cood_ver_X = sorted(lst_cood_ver_X, key=take_first)  
+
+        # số đỉnh được chọn ngẫu nhiên từ
+        num_ver_c = np.random.randint(10,30)
+        bg_get = num_ver_c
+        end_get = len(lst_cood_ver_X) - num_ver_c
+        if bg_get < end_get:
+            _tmp_ver = []
+            for idx_rm in range(  bg_get):
+                _tmp_ver.append(lst_cood_ver_X[idx_rm][1])
+            for idx_rm in range(  end_get,len(lst_cood_ver_X)  ):
+                _tmp_ver.append(lst_cood_ver_X[idx_rm][1])
+
+
+        idx_w = 0
+        while idx_w < len(_tmp_ver):
+            if _tmp_ver[idx_w] in  lst_vertices_picked:
+                _tmp_ver.pop(idx_w)
+            else:
+                lst_vertices_picked.append(_tmp_ver[idx_w])
+                idx_w = idx_w+1
+
+        _tmp_ver_pkd.append(_tmp_ver)
+
+    _inc_Z = np.random.randint(20,30)
+    llZ = np.random.randint(0,1)
+    sm = len(_tmp_ver_pkd)
+    for i in range(sm):
+        for idx_ver in _tmp_ver_pkd[i]:
+            if llZ == 0:
+                mesh_pre.vertices[idx_ver][2] =  mesh_pre.vertices[idx_ver][2] + _inc_Z*(sm - i)/sm
+            else:
+                mesh_pre.vertices[idx_ver][2] =  mesh_pre.vertices[idx_ver][2] - _inc_Z*(sm - i)/sm
+
+    # newpath = path + "_" + str(index) + ".obj"
+    # mesh_pre.export(newpath)
+
+    if index != None:
+        newpath = path + "_" + str(index) + ".obj"
+        mesh_pre.export(newpath)
+    elif path != None:
+        newpath = path + ".obj"
+        mesh_pre.export(newpath)
+
+    return mesh_pre
+
 
 
 def scarCreatingV2(mesh, path, numGenCirScar=10):
     """
-        scarCreating
+        scarCreating version 2
     """
     list_face_of_vertice = func_ListFaceOfVertice(mesh)
     limitY_min, limitY_max = func_EyeSpace(mesh.vertices)
     
-    _ = [
-        circleScar(mesh, path, list_face_of_vertice, limitY_min, limitY_max, i)
-        for i in range(numGenCirScar)
+    circleScarMeshList = [
+        circleScar(mesh, index, path, list_face_of_vertice, limitY_min, limitY_max)
+        for index in range(numGenCirScar/2)
         ]
 
-    return True
\ No newline at end of file
+    longScarMeshList = [
+        longScar(mesh, index, path, list_face_of_vertice, limitY_min, limitY_max)
+        for index in range(numGenCirScar/2, numGenCirScar)
+        ]
+
+    return circleScarMeshList, longScarMeshList
\ No newline at end of file
